
global START_TEST_COMMAND, TEST_STEP_COMMAND, END_TEST_COMMAND,START_STEP_COMMAND, END_STEP_COMMAND, ASSERT_COMMAND,\
    executionReportUrl

START_TEST_COMMAND = 'mobile:test:start'
TEST_STEP_COMMAND = "mobile:test:step"
START_STEP_COMMAND = 'mobile:step:start'
END_STEP_COMMAND = "mobile:step:end"
END_TEST_COMMAND = "mobile:test:end"
ASSERT_COMMAND = "mobile:status:assert"

executionReportUrl = 'testGridReportUrl'


class TestResult:
    def __init__(self):
        pass

    def is_successful(self):
        raise NotImplementedError

    def get_message(self):
        raise NotImplementedError


class TestResultSuccess(TestResult):
    status = 'PASSED'

    def is_successful(self):
        return True

    def get_message(self):
        return self.status


class TestResultFailure(TestResult):
    status = 'FAILED'

    def __init__(self, reason, exception=None, failureReason=None):

        if exception is not None:
            self.message = str(reason) + ': ' + str(exception)
        else:
            self.message = reason

        if len(self.message) > 4096:
            self.message = self.message[:4096]
        
        if failureReason is not None:
            self.failureReason = failureReason
        else:
            self.failureReason = None

    def is_successful(self):
        return False

    def get_message(self):
        return self.message

    def get_failureReason(self):
        return self.failureReason



class PerfectoExecutionContext:
    def __init__(self, webdriver, tags=None, job=None, project=None, customFields=[]):
        if webdriver is None:
            raise 'Missing required webdriver argument. Call your builder\'s withWebDriver() method'
        self.webdriver = webdriver
        self.job = job
        self.project = project
        clean_tags = []
        if tags is not None:
            for tag in tags:
                if not isinstance(tag, basestring):
                    print 'Warnning: perfecto tags must be strings, please change your tag:' + str(tag)
                else:
                    clean_tags.append(tag)
        self.context_tags = clean_tags
        self.customFields = {}
        if (len(customFields) > 0):
            for cf in customFields:
                self.customFields.update(cf.dict)



from constants import *
from PerfectoDeprecator import deprecated


class PerfectoReportiumClient:
    """
    Perfecto Reporting client
    """
    def __init__(self, perfecto_execution_context):
        """
        initialize a new reporting client
        :param perfecto_execution_context:
        """
        self.perfecto_execution_context = perfecto_execution_context
        self.webdriver = perfecto_execution_context.webdriver
        self.started = False

    def test_start(self, name, context):
        """
        inform reporting system about new test execution
        :param name: test name
        :param context: a context instance contains array of string tags
        :return: ---
        """
        params = {}
        job = self.perfecto_execution_context.job
        project = self.perfecto_execution_context.project
        test_tags = self.perfecto_execution_context.context_tags
        cFields = self.perfecto_execution_context.customFields
		

        if job is not None:
            params['jobName'] = job.name
            params['jobNumber'] = job.number
            params['jobBranch'] = job.branch

        if project is not None:
            params['projectName'] = project.name
            params['projectVersion'] = project.version

        params['name'] = name
        allTags = list(context.tags)

        if test_tags is not None:
            allTags.extend(test_tags)

        params['tags'] = allTags
	
        if (len(context.customFields) > 0):
            cFields.update(context.customFields)
		
        """
        Need to convert the Custom fields into a string parameter
        where each Custom Field is in the form <key>=<value>
        """
        str_cFields = []
        for key, val in cFields.items():
            cFieldS = key + '=' + val 
            str_cFields.append(cFieldS)

        params['customFields'] = str_cFields
        self.execute_script(START_TEST_COMMAND, params)
        self.started = True

    @deprecated('Consider to use step_start and step_end commands instead')
    def test_step(self, description):
        """
        ### Deprecated call use step_start instead ###
        inform reporting system about a new test step
        :param description:
        :return:
        """
        params = {'name': description}
        self.execute_script(START_STEP_COMMAND, params)

    def step_start(self, description):
        """
        inform reporting system about a new test step
        :param description:
        :return:
        """
        params = {'name': description}
        self.execute_script(START_STEP_COMMAND, params)

    def step_end(self, message=None):
        """
        inform reporting system about an end of a test step
        :param message:
        :return:
        """
        params = {'message': message}
        self.execute_script(END_STEP_COMMAND, params)


    def test_stop(self, test_result,context=None):
        """
        inform reporting system about an end of test execution
        :param test_result:
        :return:
        """

        if not self.started:
            return False
        params = {'success': test_result.is_successful()}

        if test_result.is_successful() is False:
            params['failureDescription'] = test_result.get_message()
            params['failureReason'] = test_result.get_failureReason()
        if context != None: #if there is no context skip to the end
            if context.tags is not None:
                allTags = list(context.tags)
                params['tags'] = allTags

            if (len(context.customFields) > 0):
                cFields = context.customFields
                """
                Need to convert the Custom fields into a string parameter
                where each Custom Field is in the form <key>=<value>
                """
                str_cFields = []
                for key, val in cFields.items():
                    cFieldS = key + '=' + val
                    str_cFields.append(cFieldS)

                params['customFields'] = str_cFields
        self.execute_script(END_TEST_COMMAND, params)
        return True

    def reportium_assert(self, message, status):
        """
        reportium assertion
        :param message:
        :param status:
        :return:
        """
        params = {
            'message': message,
            'status': status
        }

        self.execute_script(ASSERT_COMMAND, params)

    def report_url(self):
        """
        retrieve the report url
        :return: report's url
        """
        url = ''
        if hasattr(self.webdriver, 'capabilities'):
            url = str(self.webdriver.capabilities[executionReportUrl])
        else:
            raise 'WebDriver instance is assumed to have Selenium Capabilities'
        return url

    def execute_script(self, script, params):
        """
        Using WebDriver instance in order to execute command against reporting system
        :param script: string representation of the command to be executed
        :param params: commands parameters
        :return: command's return value
        """
        return self.webdriver.execute_script(script, params)





class TestContext:

    def __init__(self, customFields = [], tags=None):
        self.tags = tags
        self.customFields = {}
        if (len(customFields) > 0):
            for cf in customFields:
                self.customFields.update(cf.dict)



class Project:
    def __init__(self, name, version):
        self.name = name
        self.version = version


class Job:
    def __init__(self, name, number=0, branch=None):
        self.name = name
        self.number = number
        self.branch = branch

	
class CustomField:
    def __init__(self, key, value):
        self.dict = {key: value}
        self.value = value
	self.key = key



import warnings
import functools


def deprecated(extra_info=None):
    """
    deprecator's wrapper for extra arguments
    :param extra_info: extra information about the method deprecation
    :return: real deprecator method
    """

    def real_deprecated(func):
        """
        Generic Perfecto deprecator uses to warning about depreceted
        function call. Wrap a function using this deprecator to extra information
        about replacemnet for function {func}.
        :param func: function passed to the decorator
        :return: wrapper function
        """
        @functools.wraps(func)
        def func_deprecated(*args, **kwargs):
            deprecated_msg = "Call to deprecated function \"{}\"".format(func.__name__)
            warnings.simplefilter('always', DeprecationWarning)  # Shows only deprecation warnings for a while

            # if provided extra info add it to the deprecation message
            if extra_info is not None:
                deprecated_msg += '\n' + extra_info

            warnings.warn(deprecated_msg, category=DeprecationWarning, stacklevel=2) # warn
            warnings.simplefilter('default', DeprecationWarning)  # reset filter back
            return func(*args, **kwargs)

        return func_deprecated

    return real_deprecated



from distutils.core import setup
release_version = '3.0.11'

setup(
    name='perfecto',
    packages=['perfecto','perfecto/client', 'perfecto/Exceptions', 'perfecto/model', 'perfecto/test'],  # this must be the same as the name above
    version=release_version,
    description='Perfecto Reporting SDK for Python\nPerfecto Reporting is a multiple execution digital report, that enables quick navigation within your latest build execution. Get visibility of your test execution status and quickly identify potential problems with an aggregated report. Hone-in and quickly explore your test results all within customized views, that include logical steps and synced artifacts. Distinguish between test methods within a long execution. Add personalized logical steps and tags according to your team and organization.\n For release notes see: https://github.com/PerfectoCode/Samples/blob/master/Reporting/README.md',
    author='Perfecto',
    author_email='perfecto@perfectomobile.com',
    url='https://github.com/PerfectoCode',  # use the URL to the GitHub repo
    download_url='https://github.com/PerfectoCode',
    keywords=['Perfecto', 'PerfectoMobile', 'Reporting', 'Selenium', 'Appium', 'Automation testing'],
    classifiers=[ "Programming Language :: Python :: 2",
        "License :: OSI Approved :: Apache Software License 2.0 (Apache-2.0)",
        "Operating System :: OS Independent"],
)



from TestResult import TestResultSuccess, TestResultFailure


class TestResultFactory:
    @staticmethod
    def create_success():
        return TestResultSuccess()

    @staticmethod
    def create_failure(reason, traceback=None, failureReason=None):
        return TestResultFailure(reason, traceback, failureReason)

